package me.icymint.libra.sas.session;

import java.io.File;
import java.util.concurrent.SynchronousQueue;
import java.util.concurrent.atomic.AtomicBoolean;

import me.icymint.libra.sas.SasExecutorService;
import me.icymint.libra.sas.config.SasConfig;
import me.icymint.libra.sas.config.SasSystemConfig;
import me.icymint.libra.sas.exception.SasAbortException;
import me.icymint.libra.sas.exception.SasCacheFileLockedException;
import me.icymint.libra.sas.exception.SasInterruptException;
import me.icymint.libra.sas.exception.SasNotRunException;
import me.icymint.libra.sas.exception.SasRunErrorException;
import me.icymint.libra.sas.task.SasSubmit;
import me.icymint.libra.sas.task.TaskConfig;
import me.icymint.libra.sas.task.TaskScanner;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class SasSession implements Runnable {
	protected final Logger logger = LoggerFactory.getLogger(getClass());
	private static File CACHE;
	private final SynchronousQueue<SasExecutorService> sq = new SynchronousQueue<SasExecutorService>();

	static {
		SasSystemConfig ssc = SasConfig.SAS_SYSTEM_CONFIG;
		CACHE = new File(ssc.getProperty("G_ROOT") + File.separator
				+ ssc.getProperty("G_LIB_DIR"), ssc.getProperty("G_CACHE"));
		CACHE.mkdirs();
	}

	private SessionScanner ss;
	private boolean flag = false;
	private AtomicBoolean wait = new AtomicBoolean();
	private AtomicBoolean task = new AtomicBoolean(false);

	public SasSession() {
		File f = getSessionCache();
		ss = new SessionScanner(f, f.getName()) {
			@Override
			public void invoke(SasConfig config) throws InterruptedException,
					SasInterruptException, SasRunErrorException,
					SasAbortException, SasCacheFileLockedException {
				// while (!SasSession.this.isShutdown() || task.get()) {

				if (task.get()) {
					task.set(false);
					try {
						wait.set(true);
						SasExecutorService task;
						synchronized (wait) {
							// System.out.print("wait");
							task = sq.take();
							// System.out.println("--end");
							wait.set(false);
						}
						task.execute();
					} catch (SasNotRunException e) {
						logger.error("奇怪的事情发生了。");
					}
					if (SasSession.this.isShutdown()) {
						config.shutdown();
					}
				}
				// }
			}
		};
	}

	public void execute() throws SasRunErrorException, SasInterruptException,
			SasAbortException, SasNotRunException {
		try {
			ss.execute();
		} catch (SasCacheFileLockedException e) {
			logger.error("SAS缓存目录被占用了!");
			throw new SasNotRunException();
		}
	}

	public String getName() {
		return ss.getName();
	}

	private File getSessionCache() {
		File cache;
		int i = 0;
		synchronized (CACHE) {
			do {
				cache = new File(CACHE, "SAS会话-" + i++);
			} while (cache.exists());
			cache.mkdirs();
		}
		return cache;
	}

	public TaskScanner<String, TaskConfig> invoke(String sas)
			throws InterruptedException {
		SasSubmit ssx = new SasSubmit(ss.getConfig(), sas);
		invoke(ssx);
		return ssx;
	}

	protected <V, T extends TaskConfig> void invoke(TaskScanner<V, T> ts)
			throws InterruptedException {
		while (task.get()) {
			Thread.sleep(10);
		}
		task.set(true);
		sq.put(ts);
	}

	public boolean isShutdown() {
		return task.get() ? false : flag;
	}

	@Override
	public void run() {
		ss.run();
	}

	public void shutdown() {
		flag = true;
		ss.shutdown();
	}
}
