package me.icymint.libra.sas.task;

import java.io.IOException;

import me.icymint.libra.sas.exception.SasAbortException;
import me.icymint.libra.sas.exception.SasCacheFileLockedException;
import me.icymint.libra.sas.exception.SasInterruptException;
import me.icymint.libra.sas.exception.SasNotRunException;
import me.icymint.libra.sas.exception.SasRunErrorException;
import me.icymint.libra.sas.scanner.AbstractScanner;
import me.icymint.libra.sas.session.SessionConfig;

import org.apache.commons.io.FileUtils;

public abstract class TaskScanner<V, T extends TaskConfig> extends
		AbstractScanner<V, T> {
	private final String sas;
	private final SessionConfig sc;

	public TaskScanner(SessionConfig sc, String sas, T tc) {
		super(tc);
		this.sas = sas;
		this.sc = sc;
	}

	@Override
	public void finallizeScanner(T config) throws SasRunErrorException {
		while (this.getSessionConfig().getCmdFile().exists()) {
			synchronized (this) {
				try {
					this.wait(10);
				} catch (InterruptedException e) {
				}
			}
		}
		if (config.getErrorLog().exists()) {
			try {
				FileUtils.copyFile(config.getErrorLog(), config.getErrorFlag());
			} catch (IOException e) {
			}
		}
		super.finallizeScanner(config);
	}

	public SessionConfig getSessionConfig() {
		return sc;
	}

	@Override
	public void initializeScanner(T config) throws SasCacheFileLockedException {
		try {
			super.initializeScanner(config);
			config.createStartFlag();
		} catch (SasNotRunException e) {
			logger.error("奇怪的事情发生了。");
		}
	}

	@Override
	public String sas() {
		return sas;
	}

	@Override
	public void scan(TaskConfig config) throws SasInterruptException,
			SasRunErrorException, SasAbortException {
		if (this.getSessionConfig().isShutdown()) {
			config.shutdown();
		}
		if (config.isStopped())
			config.createFinishFlag();
	}
}
